@@ -28,14 +28,13 @@ $wp->set(align => 'left');
 $gd->rectangle($wp->get_bounds(10,10), $blue);
 $wp->draw(10,10);
 
-$wp->set_font('cetus.ttf', 10);
+$wp->set_font('cetus', 10);
 $wp->set(align => 'justified', line_space => 0);
 $gd->rectangle($wp->get_bounds(210,10), $blue);
 $wp->draw(210,10);
 
 $wp->font_path('/usr/share/fonts/ttfonts');
-$wp->set_font(['Arialn.ttf', 'cetus.ttf'], 10);
-
+$wp->set_font(['Arialn', 'cetus'], 10);
 $wp->set(align => 'right');
 $gd->rectangle($wp->get_bounds(10,120), $blue);
 $wp->draw(10,120);
